use strict;
use warnings;

use Test::More tests => 1;                      # last test to print

BEGIN { use_ok( 'WebService::NotifyMyAndroid' ); }

use Data::Dumper;

my $APIKEY = '67f9b6dbd5cc00f245ff677e288f4e3aecc97f49cbf3b2a3';

# this test is a stub; expand it later
